/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include "version.h"
#include "observ_table.h"
#include "toska.h"
#include "my_stdio.h"



char *help_text[] ={
	"  -b\t\t - set base directoy (def.: "BASE_DIR")",
//	"  -d\t\t - set data directoy (def.: base_dir/"DATA_DIR")",
	"  -m\t\t - set modes definition file (def.: base_dir/"MODES_FILE")",
	"  -s\t\t - set sequence file (def.: base_dir/"SEQUENCE_FILE")",
	"  -S\t\t - output sequence list.",
	"  -V\t\t - show program version.",
	"  --help\t - this help !!!.",
	""
};


void version(char *program_name)
{
	fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
	fprintf(stderr,
		"Usage: \t%s [-hbmsS][--help]\n",
		program_name);
}


void help()
{
	int i=0;

	while((char) *help_text[i] != 0)
		fprintf(stderr,"%s\n",help_text[i++]);
	fprintf(stderr,"\n");
}


int sec2hms(char *str,time_t sec)
{
	int h,m,s;
	
	h = sec / 3600;
	s = sec - h * 3600;
	m = s / 60;
	s = s - m * 60;
	sprintf(str, "%02d:%02d:%02d", h, m, s);

	return 0;
}

int main(int argc, char **argv)
{
	int i;
	struct modes *obs, *obsp;
	struct sequence *seq, *seqp;
	int loop_inc[SEQ_MAX_LOOP];
	int loop;
	int count;
	int countt = 0;
	char base_dir[_POSIX_PATH_MAX];
	char data_dir[_POSIX_PATH_MAX];
	char modes_file[_POSIX_PATH_MAX];
	char sequence_file[_POSIX_PATH_MAX];
	char *program_name;
	int out_sequence_flag = 0;
	char hms[16];
	char thms[16];
	time_t secs, tsecs, tsecs_m[] = {0, 0, 0};
	char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 101},
		{0,  0,  0,  0}
	};

	base_dir[0] = 0;
	data_dir[0] = 0;
	modes_file[0] = 0;
	sequence_file[0] = 0;

	/* who is my name? */
	if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
	else program_name  = argv[0];
  
	verbose_flag = 1;
	opterr =  0;
	while  ((op = getopt_long(argc, argv, "Vvb:m:f:s:S",
				  longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			INCREMENT_VERBOSE_FLAG();
			/* verbose_flag = (verbose_flag << 1) | 1; */
			break;
		case 101:
			usage(program_name);
			help();
			exit(-1);
			break;
		case 'b':
			sprintf(base_dir, "%s", optarg);
			if(base_dir[strlen(base_dir) - 1] == '/' )
				base_dir[strlen(base_dir) - 1] = 0;
			break;
		case 'd':
			sprintf(data_dir, "%s", optarg);
			if(data_dir[strlen(data_dir) - 1] == '/' )
				data_dir[strlen(data_dir) - 1] = 0;
			break;
		case 'f':
		case 'm':
			sprintf(modes_file, "%s", optarg);
			break;
		case 's':
			sprintf(sequence_file, "%s", optarg);
			break;
		case 'S':
			out_sequence_flag = 1;
			break;
		case 'V':
			version(program_name);
			exit(-1);
			break;
		default:
			usage(program_name);
			exit(-1);
		}

	if (base_dir[0] == 0) {
		sprintf(base_dir, "%s", BASE_DIR);
	}
	if (data_dir[0] == 0) {
		sprintf(data_dir, "%s/%s", base_dir, DATA_DIR);
	}
	if (modes_file[0] == 0) {
		sprintf(modes_file, "%s/%s", base_dir, MODES_FILE);
	}
	if (sequence_file[0] == 0) {
		sprintf(sequence_file, "%s/%s", base_dir, SEQUENCE_FILE);
	}

	PVERB(PINFO,"Reading modes file: %s\n", modes_file);
	obs = obsp = get_modes_table(modes_file);
	if( obs == NULL){
		PVERB(PERR, "Error opening modes file: %s.\n",
		       modes_file );
		exit(1);
	}
	PVERB(PINFO,"Reading sequence file: %s\n", sequence_file);
	seq = seqp = get_sequence_table(sequence_file);
	if( seq == NULL){
		PVERB(PERR, "Error opening sequence file: %s.\n",
		       sequence_file );
		exit(1);
	}
	if (out_sequence_flag) {
		for(i=0;i<SEQ_MAX_LOOP;i++) loop_inc[i] = 1;

		printf("#M Pa Az Pol Time: Secuence Acu.    lev files\n");
		while (seqp != NULL ) {
			if( seqp->type == SEQ_TYPE_DATA) {
				loop = 1;
				loop_inc[seqp->level] = seqp->loop;
				for(i=0;i<=seqp->level;i++) 
					loop *= loop_inc[i];
				time_estimation_mode(seqp->mode, obs, 
						     seqp->partition, 
						     &count, &secs);
				count *= loop;
				countt += count;
				tsecs_m[seqp->mode] += secs * (time_t) loop;
				sec2hms(hms, secs);
				sec2hms(thms, 
					tsecs_m[0] + tsecs_m[1] + tsecs_m[2]);
				//tsecs_m[seqp->mode]);
				printf("%1d %2d %03d %c  %6d %8s %8s %1d %6d\n", 
				       seqp->mode, seqp->partition, 
				       seqp->azimuth, 
				       seqp->polarization, (int) secs, 
				       hms, thms, 
				       
				       seqp->level, count);
			}else
			if (seqp->type == SEQ_TYPE_RESTART)
				printf("# restart.\n");
			seqp = seqp->next;

		}
		for(i=0; i<3; i++){
			tsecs = tsecs_m[i];
			sec2hms(thms, tsecs);
			printf("# Mode %d : %6ld %s\n", i, tsecs, thms);
		}
		tsecs = tsecs_m[0] + tsecs_m[1] + tsecs_m[2];
		sec2hms(thms, tsecs);
		printf("# Total  : %6ld %s %7d files\n", tsecs, thms, countt);

	}else {
		printf("#M StartFre StopFreq RBW     Swee  R S     RL   VBW   LG\n");
		while( obsp != NULL){
			printf("%d %8.5f %8.5f %8.3f %4d %2d %c %6.1f %7.2f %4.1f\n", 
			       obsp->mode, obsp->start_freq, obsp->stop_freq,
			       obsp->RBW, obsp->sweep, obsp->repetitions, 
			       obsp->switchs,
			       obsp->referance_level, obsp->VBW, 
			       obsp->disp_escale);
			obsp = obsp->next;
		}
	}
	free_modes_table(obs);
	free_sequence_table(seq);

	return 0;
}

