/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#ifndef _OBSERV_TABLR_H
#define _OBSERV_TABLR_H

#include <time.h>
#include <limits.h>

/* 2.9866 */
#define SOME_LOST_TIME 1.57 /* lost time in wating and reading data */

#define SEQ_MAX_LOOP 10

#define SEQ_TYPE_NULL    0
#define SEQ_TYPE_DATA    1
#define SEQ_TYPE_LOOP    2
#define SEQ_TYPE_POOL    3
#define SEQ_TYPE_RESTART 4


struct modes {
	int mode;
	double start_freq;
	double stop_freq;
	double RBW;
	int sweep;
	int sweep_obs;
	int repetitions;
	char switchs;
	double referance_level; /* added 14/07/2005 v: 0.0.6 */ 
	double VBW; /* added 14/07/2005 v: 0.0.6 */ 
	double disp_escale; /* added 14/07/2005 v: 0.0.6 */
	struct modes *next;
};


enum polarization_def {
	VERTICAL = 'v',
	HORIZONTAL = 'h',
	INDEF = '?'	
};


struct sequence {
	int type;
	int level;
	int loop;
	int azimuth;
	char polarization;
	int mode;
	int partition;
	struct sequence *next;
};


struct obs_setup {
	time_t start;
	char *base_dir;
	char *data_dir;
	int azimuth;
	char polarization;
	int mode;
	int partition;
	int current_rep;
	int naxis1;
	int fd_ther; /* themometer device */
	double temperature;
	double *data;
	int fd_rot;
	char file[_POSIX_PATH_MAX];
	struct modes *obs;
};


struct modes *get_modes_table(char *file_name);
int free_modes_table(struct modes *obs);

struct sequence *get_sequence_table(char *file_name);
int free_sequence_table(struct sequence *seq);

int time_estimation_mode(int mode, struct modes *mod, int partition, 
			    int *count, time_t *time_es);


#endif /* ifndef _OBSERV_TABLR_H */
